"""
Define interface to implement in order to create Vault plugins.
"""
import abc
import os
import datetime
import aiofiles


class VaultABC(abc.ABC):
    """
    Abstract base class (interface) that should be implemented
    to create a DocsVault plugin.
    """

    @abc.abstractmethod
    def create_directory(self, _path):
        """
        Create given directory in the vault.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def delete_directory(self, _path, _file):
        """
        Remove directory from the vault.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def browse_directory(self, _path):
        """
        Return content of the vault directory.
        """

    @abc.abstractmethod
    async def create_file(self, _path, _file):
        """
        Write ciphered file in the vault.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def delete_file(self, _path, _file):
        """
        Delete ciphered file from the vault.
        """
        raise NotImplementedError


class LocalVault(VaultABC):
    """
    Plugin to interract with a local vault (local folder)
    """

    def __init__(self, _path):
        self.path = _path

    def _relative_to_vault_path(self, _path):
        return os.path.join(self.path, '' if _path is None else _path)

    def create_directory(self, _path):
        os.makedirs(self._relative_to_vault_path(_path))
        return {'success': True, 'message': 'Directory successfully created!'}

    def delete_directory(self, _path, _file):
        return NotImplemented

    def browse_directory(self, _path):
        real_path = self._relative_to_vault_path(_path)

        content = []

        for filename in os.listdir(real_path):
            content.append({
                'filename': filename,
                'relpath': os.path.join(_path, filename),
                'created': datetime.datetime.fromtimestamp(
                    os.path.getmtime(os.path.join(real_path, filename))
                    ).strftime('%Y/%m/%d %H:%M:%S'),
                'filetype': 'folder' if os.path.isdir(os.path.join(real_path, filename)) else 'file'
            })

        return content

    async def create_file(self, _path, _file):
        directory = self._relative_to_vault_path(_path)

        async with aiofiles.open(os.path.join(directory, _file.name), 'wb') as file:
            await file.write(_file.body)

        return {'success': True, 'message': 'Stored in vault!'}

    def delete_file(self, _path, _file):
        return NotImplemented


if __name__ == '__main__':
    test_vault = LocalVault('vault')
    test_content = test_vault.browse_directory('credentials')
    print(test_content)



"""
Define endpoints for docsvault's API.
"""
from sanic import response, Blueprint

from .settings import settings
from .vault import LocalVault


api = Blueprint('api')
vault = LocalVault(settings.get('local_vault'))


@api.get('/browse')
@api.get('/browse/<_path:path>')
async def api_browse_vault(_request, _path=''):
    """
    Return content of given directory
    """
    result = vault.browse_directory(_path)
    return response.json(result)


@api.post('/vault/upload')
@api.post('/vault/upload/<_path:path>')
async def upload_file(request, _path=None):
    """
    Send file and upload it to the vault
    """
    result = await vault.create_file(_path, request.files["file"][0])
    return response.json(result)


@api.post('/vault/directory/<_path:path>')
async def create_directory(_request, _path):
    """
    Create a new directory in the vault
    """
    result = vault.create_directory(_path)
    return response.json(result)



"""
Create sanic ASGI web application that is launched in debug
mode if this script is called directly.
"""
from sanic import Sanic
from sanic_cors import CORS

from .webui import webui
from .api import api

app = Sanic()
CORS(app)

app.register_blueprint(webui)
app.register_blueprint(api, url_prefix='/api')


if __name__ == "__main__":
    app.run(host="0.0.0.0", port=8000)



"""
Deprecated, will be deleted in profit of an implementation of
a LocalVault class that implement VaultABC.
"""
import os
from datetime import datetime
from .settings import settings


def browse_vault(rel_path=''):
    """
    Retrieve vault content of given directory
    """
    content = []

    if not rel_path:
        real_path = settings.get('local_vault')
    else:
        real_path = os.path.join(settings.get('local_vault'), rel_path)

    for filename in os.listdir(real_path):

        content.append({
            'filename': filename,
            'relpath': os.path.join(rel_path, filename),
            'created': datetime.fromtimestamp(
                os.path.getmtime(os.path.join(real_path, filename))).strftime('%Y/%m/%d %H:%M:%S'),
            'filetype': 'folder' if os.path.isdir(os.path.join(real_path, filename)) else 'file'
        })

    return content



"""
Middleware object used to abstract interractions with a
local settings file for docsvault.
"""
import uuid

UNDEFINED = uuid.uuid4()


class Settings:
    """
    Object used to interract with local settings file.
    """

    def __init__(self):
        """
        Load settings from local settings file
        """
        self.settings = {
            'local_vault': 'vault',
            'remote_vault': None
        }

    def get(self, key: str, default=UNDEFINED):
        """
        Extract value from the configuration file.

        Positional arguments:
            key -- (str) Path of the key in json notation
            default -- (any) Value to return if key is not found in settings

        Raises:
            KeyEror -- If value is not found and default value is not provided
        """
        value = self.settings.get(key, default)

        if value is UNDEFINED:
            raise KeyError('{} not found in settings'.format(key))

        return value

    def set(self, key: str, value: object):
        """
        Edit value of a setting.
        """


settings = Settings()



import setuptools

setuptools.setup(
    name='docsvault',
    version='0.1.1',
    author='Sylvan Le Deunff',
    author_email='sledeunf@gmail.com',
    description='Web app used to securely version your documents on git',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        
        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='docsvault, documents, store, vault',

    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'dvault = docsvault.__main__:main',
        ],
    },

    project_urls={
        'Bug Reports': 'https://github.com/sylvan-le-deunff/docsvault/issues',
        'Source': 'https://github.com/sylvan-le-deunff/docsvault',
        #'Funding': 'https://donate.pypi.org',
        #'Say Thanks!': 'http://saythanks.io/to/example',
    },
)



"""
Define groups of routes to serve static content.
"""
from sanic import Blueprint


webui = Blueprint('webui')

FRONTEND_PATH = 'docsvault/static'

webui.static('/', FRONTEND_PATH + '/index.html')
webui.static('/favicon.ico', FRONTEND_PATH + '/favicon.ico')

webui.static('/css', FRONTEND_PATH + '/css')
webui.static('/img', FRONTEND_PATH + '/img')
webui.static('/js', FRONTEND_PATH + '/js')



import sys, os
import argparse
import webbrowser
import subprocess


def parse_args(args=None):    
    parser = argparse.ArgumentParser('docsvault')
    subparsers = parser.add_subparsers(dest='command')

    server_parser = subparsers.add_parser('server', help='control docsvault server state')
    server_parser.add_argument('--host', type=str, default='localhost', help='ip or name on which server listen')
    server_parser.add_argument('--port', type=str, default='8024', help='port on which server listen')
    server_parser.add_argument('--debug', action='store_true', help='if set, enable hot reload')
   
    webui_parser = subparsers.add_parser('webui', help='open docsvault web ui in your browser')

    args = parser.parse_args(args)
    if args.command is None:
        parser.print_help()
        sys.exit(0)
    
    return args


def start_server(host='localhost', port=8025, debug=False):
    python_bin = os.path.dirname(sys.executable)

    if debug:
        command = [
            os.path.join(python_bin, 'uvicorn'),
            'docsvault.wsgi:app', '--reload',
            '--port', str(port)
        ]
        print(command)
    else:
        command = [
            os.path.join(python_bin, 'gunicorn'),
            'docsvault.wsgi:app',
            '--bind', '{}:{}'.format(host, port),
            '-k', 'uvicorn.workers.UvicornWorker'
        ]
        if not debug:
            command.append('--daemon')
            print('Server listening on http://{}:{}'.format(host, port))
        print(command)

    subprocess.call(command)


def main(args=None):
    """
    Dispatch parsed command to the
    corresponding docsvault function.
    """
    args = parse_args(args)
    
    if args.command == 'server':
        print(args.port, args.host)
        start_server(host=args.host, port=args.port)
    elif args.command == 'webui':
        webbrowser.open('http://localhost:8025')  


if __name__ == '__main__':
    main()
